@@ -330,7 +330,7 @@ dbartsSampler <-
 
                   invisible(NULL)
                 },
-                setOffset = function(offset, updateState = NA) {
+                setOffset = function(offset, updateScale = FALSE, updateState = NA) {
                   'Changes the offset slot used to adjust the response.'
                   ptr <- getPointer()
                   selfEnv <- parent.env(environment())
@@ -364,7 +364,7 @@ dbartsSampler <-
                   }
 
                   selfEnv$data@offset <- offset
-                  .Call(C_dbarts_setOffset, ptr, data@offset)
+                  .Call(C_dbarts_setOffset, ptr, data@offset, updateScale)
                   if (!identical(offset.test, NA)) {
                     selfEnv$data@offset.test <- offset.test
                     .Call(C_dbarts_setTestOffset, ptr, data@offset.test)
@@ -388,6 +388,16 @@ dbartsSampler <-
 
                   invisible(NULL)
                 },
+                setSigma = function(sigma, updateState = NA) {
+                  'Changes the residual standard deviation parameter for each chain.'
+                  ptr <- getPointer()
+                   
+                  .Call(C_dbarts_setSigma, ptr, sigma)
+                  if ((is.na(updateState) && control@updateState == TRUE) || identical(updateState, TRUE))
+                    storeState(ptr)
+
+                  invisible(NULL)
+                },
                 setPredictor = function(x, column, forceUpdate, updateCutPoints = FALSE, updateState = NA) {
                   'Changes a single column of the predictor matrix, or the entire matrix itself if the column argument is missing. Can force the trees to update, or roll back for rejection sampling.'
                   
