@@ -1,6 +1,8 @@
 #ifndef DBARTS_RANDOM_HPP
 #define DBARTS_RANDOM_HPP
 
+struct ext_rng;
+
 namespace dbarts {
   typedef enum {
     RNG_ALGORITHM_WICHMANN_HILL = 0,
@@ -11,7 +13,8 @@ namespace dbarts {
     RNG_ALGORITHM_USER_UNIFORM,
     RNG_ALGORITHM_KNUTH_TAOCP2,
     RNG_ALGORITHM_LECUYER_CMRG,
-    RNG_ALGORITHM_INVALID // must be last
+    RNG_ALGORITHM_INVALID,       // here and up must match ext/random.h
+    RNG_ALGORITHM_USER_POINTER   // implies user will set rng after object is created
   } rng_algorithm_t;
   
   typedef enum {
