@@ -296,7 +296,8 @@ namespace {
     DEF_FUNC("dbarts_getTrees", getTrees, 4),
     DEF_FUNC("dbarts_predict", predict, 3),
     DEF_FUNC("dbarts_setResponse", setResponse, 2),
-    DEF_FUNC("dbarts_setOffset", setOffset, 2),
+    DEF_FUNC("dbarts_setOffset", setOffset, 3),
+    DEF_FUNC("dbarts_setSigma", setSigma, 2),
     DEF_FUNC("dbarts_setWeights", setWeights, 2),
     DEF_FUNC("dbarts_setPredictor", setPredictor, 4),
     DEF_FUNC("dbarts_updatePredictor", updatePredictor, 5),
@@ -362,18 +363,36 @@ namespace {
     DEF_FUNC("initializeChiSquaredPriorFromOptions", dbarts_initializeChiSquaredPriorFromOptions),
     DEF_FUNC("invalidateChiSquaredPrior", dbarts_invalidateChiSquaredPrior),
 
+    DEF_FUNC("createControl", dbarts_createControl),
+    DEF_FUNC("initializeControl", dbarts_initializeControl),
+    DEF_FUNC("destroyControl", dbarts_destroyControl),
+    // DEF_FUNC("invalidateControl", dbarts_invalidateControl),
+    
+    DEF_FUNC("createData", dbarts_createData),
+    DEF_FUNC("initializeData", dbarts_initializeData),
+    DEF_FUNC("destroyData", dbarts_destroyData),
+    DEF_FUNC("invalidateData", dbarts_invalidateData),
+    
+    DEF_FUNC("createModel", dbarts_createModel),
+    DEF_FUNC("initializeModel", dbarts_initializeModel),
+    DEF_FUNC("destroyModel", dbarts_destroyModel),
+    DEF_FUNC("invalidateModel", dbarts_invalidateModel),
+    
     DEF_FUNC("createFit", dbarts_createFit),
     DEF_FUNC("initializeFit", dbarts_initializeFit),
     DEF_FUNC("destroyFit", dbarts_destroyFit),
     DEF_FUNC("invalidateFit", dbarts_invalidateFit),
     
+    DEF_FUNC("printInitialSummary", dbarts_printInitialSummary),
     DEF_FUNC("setRNGState", dbarts_setRNGState),
     
     DEF_FUNC("runSampler", dbarts_runSampler),
     DEF_FUNC("runSamplerForIterations", dbarts_runSamplerForIterations),
+    DEF_FUNC("runSamplerWithResults", dbarts_runSamplerWithResults),
     DEF_FUNC("sampleTreesFromPrior", dbarts_sampleTreesFromPrior),
     DEF_FUNC("setResponse", dbarts_setResponse),
     DEF_FUNC("setOffset", dbarts_setOffset),
+    DEF_FUNC("setSigma", dbarts_setSigma),
     DEF_FUNC("setPredictor", dbarts_setPredictor),
     DEF_FUNC("updatePredictor", dbarts_updatePredictor),
     DEF_FUNC("setTestPredictor", dbarts_setTestPredictor),
